/* global constants, oscObjects, log, LanguageIOError */

class Config {
  constructor(
    serverCredentials,
    serverUrl,
    agentLanguage,
    languageFieldColumnName,
    languageFieldDisplayName,
    qualityRatingType
  ) {
    this.serverCredentials = serverCredentials;
    this.serverUrl = serverUrl;

    this.agentLanguage = parseInt(agentLanguage, 10);
    this.languageFieldColumnName = languageFieldColumnName;
    this.languageFieldDisplayName = languageFieldDisplayName;
    this.translationType = "machine";

    // ROADBLOCKED BY BCO-4: autoresponder not yet available to clients
    this.sendAutoresponder = false;

    this.qualityRatingInstructions = "";
    this.qualityRatingType = qualityRatingType;
    this._qualityRatingConfigsLoaded = false;

    this.endUserFlaggedMessageText = "{customer} has <strong>flagged</strong> a translated message";
    this.endUserUnFlaggedMessageText = "{customer} has <strong>un-flagged</strong> a translated message";
    this.goToMessageText = "Go to Message";
  }

  /** @returns {Promise<int>} the id of the current customer language */
  async getCustomerLanguage() {
    return new Promise((resolve) => {
      oscObjects.extensionProvider.registerWorkspaceExtension((workspaceRecord) => {
        const name = "Incident." + this.languageFieldColumnName;
        workspaceRecord.getFieldValues([name]).then((fieldDetails) => {
          const languageId = parseInt(fieldDetails.getField(name).getValue(), 10);
          return resolve(languageId);
        });
      });
    });
  }

  async loadAgentQualityRatingConfig(lioRestManager) {
    if (this._qualityRatingConfigsLoaded) {
      return Promise.resolve();
    }
    return new Promise((resolve, reject) => {
      const instructionTypes = {
        // TODO: when the server has more rating instructions, request the one pertinent to the configuring rating type
        // ex: `QUALITY_RATING_INSTRUCTIONS_TYPE_${this.qualityRatingType}`,
        TRANSLATION_RATING_LABEL: this.qualityRatingInstructions,
        CUSTOMER_HAS_FLAGGED: this.endUserFlaggedMessageText,
        CUSTOMER_HAS_UNFLAGGED: this.endUserUnFlaggedMessageText,
        GO_TO_FLAGGED: this.goToMessageText,
      };

      lioRestManager
        .getInstructions(instructionTypes, this.agentLanguage, false)
        .then((instructions) => {
          this.qualityRatingInstructions = instructions.TRANSLATION_RATING_LABEL;
          this.endUserFlaggedMessageText = instructions.CUSTOMER_HAS_FLAGGED;
          this.endUserUnFlaggedMessageText = instructions.CUSTOMER_HAS_UNFLAGGED;
          this.goToMessageText = instructions.GO_TO_FLAGGED;
          this._qualityRatingConfigsLoaded = true;
          return resolve();
        })
        .catch((error) => reject(error.desc));
    });
  }
}

/** @returns dict containing the SCPs for this agent */
async function _getServerConfigurationProperties() {
  const configNames = ["languageioLogin", "languageioUrl", "languageField", "agentLanguage", "qualityRatingType"];
  return new Promise((resolve, reject) => {
    oscObjects.globalContext
      .getExtensionContext("LanguageIO_BUI_Chat")
      .then((extensionContext) => {
        extensionContext
          .getProperties(configNames)
          .then((collection) => {
            const configs = [];
            configNames.forEach((name) => {
              const config = collection.get(name);
              if (config) {
                configs[name] = config.getValue();
              }
            });
            return resolve(configs);
          })
          .catch((error) => reject(error.desc));
      })
      .catch((error) => reject(error.desc));
  });
}

/** @returns {Promise<Config>} the configuration object. can't use constructor b/c of async calls
 * @param {*} oscRestManager handles all REST calls to OSC
 */
/* eslint-disable-next-line no-unused-vars */ // used in main to create config object
async function createConfig(oscRestManager) {
  let configs;
  try {
    configs = await _getServerConfigurationProperties();
  } catch (error) {
    log.error(`Could not retrieve server configuration properties: ${error}`);
    return Promise.reject(
      new LanguageIOError(
        "Failed to retrieve Server Configuration Properties. Please view the browser's developer tools console or " +
          constants.contactText
      )
    );
  }

  if (configs.languageioLogin === null || configs.languageioLogin.length < 1) {
    return Promise.reject(
      new LanguageIOError(
        "The `languageioLogin` Server Configuration Property is not set up for your profile. Please " +
          constants.contactText
      )
    );
  }

  let languageFieldColumnName = "lang_id";
  let languageFieldDisplayName = "Language";
  if (configs.languageField !== "default") {
    try {
      languageFieldColumnName = "C$" + (await oscRestManager.getCustomLanguageLookupName(configs.languageField));
      languageFieldDisplayName = configs.languageField;
    } catch (error) {
      return Promise.reject(error);
    }
  }
  return Promise.resolve(
    new Config(
      configs.languageioLogin,
      configs.languageioUrl,
      configs.agentLanguage,
      languageFieldColumnName,
      languageFieldDisplayName,
      configs.qualityRatingType
    )
  );
}
