/* global constants, log, Log, LanguageIOError */

/* eslint-disable-next-line no-unused-vars */ // used in other files
class LioRestManager {
  /** handles all REST calls to LIO
   * @param {Config} config agent-profile-specific configuration options
   */
  constructor(config) {
    this._config = config;
    this._headers = {
      "Content-Type": "application/json",
      Authorization: "Basic " + this._config.serverCredentials,
    };
    this._rnResponseProjectId = 0;
  }

  /**
   * Gets custom instruction text for the given instruction type and locale
   * @param {string} instructionType the name of the instruction
   * @param {string} consumerLocaleId the identifier of the consumer locale
   * @param {boolean} [disableDefault] (optional) sets the `disableDefault` GET param
   * @param {string} [defaultInstructionContent] (optional) default instruction text if server does not return content
   * @returns {Promise<string>} custom instruction text, if it exists
   */
  async getInstruction(instructionType, consumerLocaleId, disableDefault = false, defaultInstructionContent = "") {
    const fullUrl = `${
      this._config.serverUrl
    }/linguistnow/resources/com.linguistnow.entities.consumerhasinstruction/getConsumerHasInstruction/?consumerLocaleId=${consumerLocaleId}&disableDefault=${
      disableDefault ? "true" : "false"
    }&instructionType=${instructionType}`;
    const parameters = {
      method: "GET",
      headers: this._headers,
    };
    return new Promise((resolve, reject) => {
      fetch(fullUrl, parameters)
        .then((response) => {
          if (response.ok) {
            response
              .json()
              .then((instruction) => {
                if (instruction.instructionContent) {
                  resolve(instruction.instructionContent);
                } else if (defaultInstructionContent !== "") {
                  resolve(defaultInstructionContent);
                } else {
                  reject(new LanguageIOError("No localized value for the instruction was returned."));
                }
              })
              .catch((error) => reject(error));
          } else {
            reject(response);
          }
        })
        .catch((error) => reject(error));
    });
  }

  /**
   * Gets custom instruction text for the given object of instruction types and locale
   * @param {Object} instructionTypes an object of instructions, where the keys are the names and the values are the default text
   * @param {string} consumerLocaleId the identifier of the consumer locale
   * @param {boolean} [disableDefault] (optional) sets the `disableDefault` GET param
   * @returns {Promise<Object>} object with custom instruction texts keyed by input name
   */
  async getInstructions(instructionTypes, consumerLocaleId, disableDefault = false) {
    return Promise.allSettled(
      Object.entries(instructionTypes).map(([instructionType, defaultInstructionContent]) =>
        // eslint-disable-next-line implicit-arrow-linebreak
        this.getInstruction(instructionType, consumerLocaleId, disableDefault, defaultInstructionContent)
      )
    ).then((instructionPromiseResults) => {
      const instructions = { ...instructionTypes };
      Object.keys(instructionTypes).forEach((instructionType, index) => {
        if (instructionPromiseResults[index].status === "fulfilled") {
          instructions[instructionType] = instructionPromiseResults[index].value;
        }
      });
      return instructions;
    });
  }

  /** @returns {Promise<string>} custom autoresponder text, if it exists */
  async getAutoresponder() {
    const customerLocaleId = await this._config.getCustomerLanguage();
    return this.getInstruction("CHAT_GREETING", customerLocaleId, true);
  }

  /** request an LIO translation for the message, and update the message with the translation
   * @param {Message} message agent or customer message to translate
   * @returns {Promise} the translation or a rejection
   */
  async getTranslation(message) {
    if (message.text.length < 1) {
      throw new Error("Translation cancelled: There is no content to translate");
    }

    const customerLanguage = await this._config.getCustomerLanguage();

    const translationEligibilityTypeIds = [];

    if (message.sender === constants.senders.AGENT) {
      return this._translate(message, this._config.agentLanguage, customerLanguage, translationEligibilityTypeIds);
    } else if (message.sender === constants.senders.END_USER) {
      if (message.ratingTypeId > 0) {
        translationEligibilityTypeIds.push(1);
      }
      return this._translate(message, customerLanguage, this._config.agentLanguage, translationEligibilityTypeIds);
    } else {
      // translation shouldn't be attempted for system messages
      return Promise.resolve();
    }
  }

  /**
   * creates a rating for a translation (**by the translation's ID**)
   * @see {@link https://languageio.atlassian.net/wiki/spaces/DEV/pages/2024112146/LIO+Rest+API+for+Rating}
   * @param {number} payload.translationId the ID of the LIO translation
   * @param {number} payload.ratingTypeId the type of the rating
   * @param {number} payload.ratingPoints the points/value of the rating
   * @returns
   */
  async createTranslationRating(payload) {
    return this._rateTranslation("POST", payload);
  }

  /**
   * updates the rating for a translation (**by the existing/previous rating's ID**)
   * @see {@link https://languageio.atlassian.net/wiki/spaces/DEV/pages/2024112146/LIO+Rest+API+for+Rating}
   * @param {number} payload.ratingId the ID of the LIO rating
   * @param {number} payload.ratingTypeId the type of the rating
   * @param {number} payload.ratingPoints the points/value of the rating
   * @returns
   */
  async updateTranslationRating(payload) {
    return this._rateTranslation("PUT", payload);
  }

  /**
   * wrapper for API requests to create or update a translation's rating
   * @see {@link https://languageio.atlassian.net/wiki/spaces/DEV/pages/2024112146/LIO+Rest+API+for+Rating}
   * @param {string} method the HTTP method for the fetch request
   * @param {number} [payload.translationId] the ID of the LIO translation - required for new ratings only
   * @param {number} [payload.ratingId] the ID of the LIO rating - required for existing ratings only
   * @param {number} payload.ratingTypeId the type of the rating
   * @param {number} payload.ratingPoints the points/value of the rating
   * @returns {object} the rating data (if successful)
   */
  async _rateTranslation(method, payload) {
    const fullUrl = `${this._config.serverUrl}/linguistnow/resources/v1/rating`;
    const parameters = {
      method,
      headers: this._headers,
      body: JSON.stringify(payload),
    };
    return new Promise((resolve, reject) => {
      fetch(fullUrl, parameters)
        .then((response) => {
          if (response.ok) {
            response
              .json()
              .then((rating) => {
                if (rating.error) {
                  log.error(rating.error);
                  reject(new LanguageIOError(rating.error.userMessage));
                } else {
                  resolve(rating);
                }
              })
              .catch((error) => reject(error));
          } else {
            reject(response);
          }
        })
        .catch((error) => reject(error));
    });
  }

  /** @returns {Promise} the translation or error
   * @param {Message} message the message to be translated
   * @param {int} sourceLocale the current language
   * @param {int} targetLocale the language we're translating to
   * @param {int[]} [translationEligibilityTypeIds] the retranslation and/or rating eligibility info
   */
  _translate(message, sourceLocale, targetLocale, translationEligibilityTypeIds = []) {
    const completedThreadType =
      message.sender === constants.senders.END_USER
        ? constants.threadTypes.translatedCustomer
        : constants.threadTypes.translatedAgent;
    const fullUrl = this._config.serverUrl + "/linguistnow/resources/v1/translate";
    const requestBody = {
      consumerSourceId: 0,
      contentTypeName: "chat",
      crmThreadType: completedThreadType,
      incidentId: 0,
      sourceContent: message.text,
      sourceLocale,
      targetLocale,
      textType: "html",
      translationType: this._config.translationType,
      translationEligibilityTypeIds,
    };
    if (this._rnResponseProjectId !== 0) {
      requestBody.rnResponseProjectId = this._rnResponseProjectId;
    }
    const parameters = {
      method: "POST",
      headers: this._headers,
      body: JSON.stringify(requestBody),
    };

    return (
      fetch(fullUrl, parameters)
        .then((response) => {
          if (response.ok) {
            return response.json().then((translation) => ({ translation }));
          } else {
            // if !response.ok, we authenticated THEN received an error
            return response.json().then((error) => ({ error }));
          }
        })
        /* either didn't authenticate successfully, or there was a problem with the response body */
        .catch((error) => ({ error }))
        .then(({ translation, error }) => {
          if (translation) {
            this._rnResponseProjectId = translation.rnResponseProjectId;
            message.addTranslationInfo(
              translation.translatedText,
              translation.rnResponseProjectId,
              translation.translationId,
              translation.wordCount
            );
            message.addStatus(`${constants.statuses.SUCCESS}: ${translation.statusMessage}`);
            return translation;
          } else if (error && error.error) {
            /* error.error is a server error, which will always have
          an id, userMessage, and message */
            message.addStatus(error.error.message);
            throw new Error(error.error.message);
          } else {
            /* if we didn't get an error.error back, we didn't get through the auth process. */
            const errorText =
              "Could not access Language I/O server. Verify your `languageioLogin` and `languageioUrl` Server Configuration Properties or " +
              constants.contactText;
            message.addStatus("ERROR: Translation failed: " + errorText);
            /* a failure to access the server is probably a serious configuration issue, so we want
          to do more than just show an inline status */
            Log.alert(errorText);
            throw new LanguageIOError(errorText);
          }
        })
    );
  }
}
