/* global ORACLE_SERVICE_CLOUD, Log, Message, LanguageIOError, LioRestManager, OscRestManager, agentTextEditor, constants, createConfig, delay, fixPlaintext, getCommandData, getEndUserWidgetInfo, getTranslationIdAndMessage, processVariables, removeSpaces, standardTextPopup, toggleTranslationFlagRating */

const extensionName = "Language I/O BUI Chat";
const extensionVersion = "1.1.0";

/* GLOBAL VARIABLES */
const allMessages = [];
const log = new Log();
const endUserQualityRatingConfig = {
  ratingTypeId: 0,
  ratingInstructions: "",
};

/** asynchronously-retrieved objects from the OSC BUI API, loaded in runExtension.
 *
 * Oracle uses their own version of Promises that behave slightly different than standard
 * JS Promises, and their async functions may need to be wrapped in standard Promises to avoid
 * unexpected execution order.
 *
 * Oracle's promises also do not support .finally()
 */
const oscObjects = {
  extensionProvider: null,
  globalContext: null,
  oracleChatClient: null,
};

/* asyncs for handling program-wide tasks like REST calls, loaded in runExtension */
let oscRestManager;
let lioRestManager;
let profileConfig;

/** agent and customer display names, loaded in runExtension */
const names = {
  agent: "Agent",
  setAgentName() {
    return oscRestManager.getAgentName().then((name) => {
      if (name !== undefined) {
        names.agent = name;
      }
      return Promise.resolve();
    });
  },
  customer: "Customer",
  setCustomerName() {
    return oscObjects.extensionProvider.registerWorkspaceExtension((workspaceRecord) => {
      const firstNameField = "Contact.first_name";
      const lastNameField = "Contact.last_name";
      workspaceRecord.getFieldValues([firstNameField, lastNameField]).then((fieldDetails) => {
        let fullName = fieldDetails.getField(firstNameField).getValue();
        fullName += " " + fieldDetails.getField(lastNameField).getValue();
        if (fullName !== "null null") {
          names.customer = fullName;
        }
        return Promise.resolve();
      });
    });
  },
  subject: "Subject",
};

/* INITIAL EXTENSION LOADING */

/** tracks chat and incident closing events and creates transcripts at the end */
const chatConclusionManager = {
  chatEnded: false,
  systemTranscriptSaved: false,
  lnThreadTranscriptSaved: false,
  subscribeToChatEndingEvents(currentEngagement) {
    currentEngagement.concluded(() => {
      agentTextEditor.disable();
      this.chatEnded = true;
    });
    currentEngagement.released(() => {
      this.chatEnded = true;
    });
  },
  subscribeToIncidentSaveEvents() {
    return new Promise((resolve) => {
      oscObjects.extensionProvider.registerWorkspaceExtension((workspaceRecord) => {
        workspaceRecord.getWorkGroupContext().then((workgroupContext) => {
          workgroupContext.getWorkGroupEntities().then((entities) => {
            for (let index = 0; index < entities.length; index += 1) {
              if (entities[index].getEntityType() === "Incident") {
                // get the workspace record and put a saved listener on it
                entities[index].getWorkspaceRecord().then((incidentRecord) => {
                  incidentRecord.addRecordSavingListener(this.createSystemThreadTranscript);
                  incidentRecord.addRecordSavedListener(this.createLNThreadTranscript);
                  return resolve();
                });
              }
            }
          });
        });
      });
    });
  },
  createSystemThreadTranscript(workspaceRecordParam) {
    // create single system thread with agent-lang messages
    return new ORACLE_SERVICE_CLOUD.ExtensionPromise((resolve) => {
      if (chatConclusionManager.chatEnded && !chatConclusionManager.systemTranscriptSaved) {
        const workspaceRecord = workspaceRecordParam.getWorkspaceRecord();
        const errorPrefix = "Failed to create system thread transcript: ";

        let systemThreadTranscript = "";
        allMessages.forEach((message) => {
          systemThreadTranscript += message.getTranscriptHtml();
        });

        workspaceRecord
          .getCurrentEditedThread("Note", true)
          .then((threadEntry) => {
            threadEntry.setContentType("html");
            threadEntry
              .setContent(systemThreadTranscript)
              .then(() => {
                log.info("Finished system thread transcript creation.");
                chatConclusionManager.systemTranscriptSaved = true;
                resolve();
              })
              .catch((error) => {
                log.warn(errorPrefix + error.message);
                resolve();
              });
          })
          .catch((error) => {
            log.warn(errorPrefix + error.message);
            resolve();
          });
      } else {
        log.info("Conditions were not met for transcript creation.");
        resolve();
      }
    });
  },
  createLNThreadTranscript(workspaceRecordParam) {
    return new ORACLE_SERVICE_CLOUD.ExtensionPromise((resolve) => {
      if (chatConclusionManager.chatEnded && !chatConclusionManager.lnThreadTranscriptSaved) {
        const incidentId = workspaceRecordParam.getWorkspaceRecord().getWorkspaceRecordId();
        oscRestManager.updateLNStatus(incidentId);

        const lnThreadPromises = [];
        allMessages.forEach((message) => {
          if (message.translatedText.length > 0) {
            let threadType = 4;
            if (message.sender === constants.senders.AGENT) {
              threadType = 2;
            }
            lnThreadPromises.push(oscRestManager.createLNThread(incidentId, threadType, message));
          }
        });
        Promise.all(lnThreadPromises)
          .then(() => {
            log.info("Finished LN thread transcript creation.");
            chatConclusionManager.lnThreadTranscriptSaved = true;
            resolve();
          })
          .catch((error) => {
            log.warn("Failed while creating an LN thread for the transcript: " + error.message);
            resolve();
          });
      } else {
        log.info("Conditions were not met for LNThread creation.");
        resolve();
      }
    });
  },
};

/** entry point for the extension. perform all initial loading tasks: set values for all async
 * globals, subscribe to events, and display GUI components and pre-load messages. */
/* eslint-disable-next-line no-unused-vars */ // used by init.html to initialize extension
async function runExtension() {
  log.info(`Initializing ${extensionName} ${extensionVersion}`);

  const loadingClass = "loading";
  document.body.classList.add(loadingClass);
  document.getElementById("load_button").hidden = true;
  document.getElementById("wrapper").hidden = false;
  /** hack to fix OSC's autosizing not adapting to the actual available space */
  // eslint-disable-next-line no-restricted-globals
  const extensionWrappers = parent.document.getElementsByClassName("extensionWrapper");
  // eslint-disable-next-line no-restricted-syntax
  for (const element of extensionWrappers) {
    if (element.innerHTML.includes("LanguageIO_BUI_Chat")) {
      element.setAttribute("style", "height: 100% !important; min-height: 100% !important;");
    }
  }

  /** display errors appropriately and re-show load button if anything fails */
  function abortLoading(error) {
    if (error.name === "LanguageIOError") {
      Log.alert(error.message);
    } else {
      Log.alert(
        `Could not load ${extensionName} ${extensionVersion}. See the browser's developer tools console for details.`
      );
    }
    log.error(error);
    document.getElementById("load_button").hidden = false;
    document.getElementById("wrapper").hidden = true;
    document.body.classList.remove(loadingClass);
  }

  try {
    // init async oracle objects that are used throughout the program
    oscObjects.extensionProvider = await ORACLE_SERVICE_CLOUD.extension_loader.load("Language I/O", 1);
    oscObjects.globalContext = await oscObjects.extensionProvider.getGlobalContext();
  } catch (error) {
    abortLoading(error);
  }

  _setRestManagersAndConfigs()
    .then(() => profileConfig.getCustomerLanguage())
    .then((customerLanguage) => {
      // abort if the customer locale isn't valid
      // eslint-disable-next-line no-restricted-globals
      if (isNaN(customerLanguage)) {
        return Promise.reject(
          new LanguageIOError(
            `The customer language field '${profileConfig.languageFieldDisplayName}' is not set. Please set the customer language and reload the ${extensionName} extension.`
          )
        );
      } else if (customerLanguage === profileConfig.agentLanguage) {
        return Promise.reject(
          new LanguageIOError(
            `The customer language set in the field '${profileConfig.languageFieldDisplayName}' matches the agent language in the Server Configuration Properties. Please update the customer language and reload the ${extensionName} extension.`
          )
        );
      } else {
        return lioRestManager
          .getInstruction("TRANSLATION_RATING_LABEL", customerLanguage, false)
          .then((instructions) => {
            endUserQualityRatingConfig.ratingInstructions = instructions;
          })
          .catch((error) => {
            log.warn("Failed to retrieve end-user facing translation quality rating instruction", error);
          });
      }
    })
    .then(() => Promise.all([names.setAgentName(), names.setCustomerName()]))
    .then(() => _setOracleChatClient())
    .then(() => {
      oscObjects.oracleChatClient
        .getAgentSession()
        .then((agentSession) => agentSession.getCurrentEngagement())
        .then((currentEngagement) => {
          Promise.all([
            chatConclusionManager.subscribeToIncidentSaveEvents(),
            chatConclusionManager.subscribeToChatEndingEvents(currentEngagement),
            _loadAndDisplaySubject(),
            _subscribeToNewMessages(currentEngagement),
            _loadAndDisplayPreviousMessages(currentEngagement),
            _sendAutoresponder(),
          ]).then(() => {
            // finish up display tasks
            agentTextEditor.setup(processAgentMessage);
            standardTextPopup.setup();
            // eslint-disable-next-line no-restricted-syntax
            for (const element of document.getElementsByClassName("load_text")) {
              element.hidden = true;
            }
            document.body.classList.remove(loadingClass);
          });
        });
    })
    .catch((error) => abortLoading(error));
}

function _setRestManagersAndConfigs() {
  return new Promise((resolve, reject) => {
    oscObjects.globalContext.getSessionToken().then((sessionToken) => {
      oscRestManager = new OscRestManager(sessionToken);
      createConfig(oscRestManager)
        .then((config) => {
          profileConfig = config;
          lioRestManager = new LioRestManager(profileConfig);
          profileConfig
            .loadAgentQualityRatingConfig(lioRestManager)
            .then(() => {
              allMessages.forEach((message) => {
                message.setQualityRatingConfig({
                  instructions: profileConfig.qualityRatingInstructions,
                  type: profileConfig.qualityRatingType,
                });
              });
            })
            .catch((error) => {
              log.warn("Failed to initialize quality rating configuration settings", error);
            });
          return resolve(profileConfig);
        })
        .catch((error) => reject(error));
    });
  });
}

function _setOracleChatClient() {
  return new Promise((resolve, reject) => {
    oscObjects.globalContext
      .getChatAPIInfo()
      .then((chatAPIInfo) => {
        ORACLE_SERVICE_CLOUD.scriptLoader.loadScript([chatAPIInfo.getChatAPIURL()]).then(() => {
          /* eslint-disable-next-line no-undef */ // this is defined by oracle
          oscObjects.oracleChatClient = IOracleChatClient;
          return resolve();
        });
      })
      .catch((error) => {
        log.error(error);
        return reject(
          new LanguageIOError(
            `Could not access Oracle BUI Chat API. Your site may need updating before ${extensionName} ${extensionVersion} can be used. Please ` +
              constants.contactText
          )
        );
      });
  });
}

function _loadAndDisplaySubject() {
  return oscObjects.extensionProvider.registerWorkspaceExtension((workspaceRecord) => {
    const subjectFieldName = "Incident.Subject";
    workspaceRecord.getFieldValues([subjectFieldName]).then((subjectField) => {
      const subject = subjectField.getField(subjectFieldName).getValue();
      // ignoring 'Chat Session' because it's the default value for the field.
      if (subject === "Chat Session") {
        return Promise.resolve();
      } else {
        const createdTimeFieldName = "Incident.Created";
        return workspaceRecord.getFieldValues([createdTimeFieldName]).then((createdTimeField) => {
          const createdTime = createdTimeField.getField(createdTimeFieldName).getValue();
          const subjectMessage = new Message(
            constants.senders.SUBJECT,
            createdTime,
            subject,
            constants.statuses.awaiting,
            profileConfig.qualityRatingInstructions,
            profileConfig.qualityRatingType
          );
          _displayMessage(subjectMessage, true);
          _translateAndContinue(subjectMessage);
          return Promise.resolve(); // we don't need to wait on the translation for loading
        });
      }
    });
  });
}

/** create an extension message object, display, and attempt translation for any new messages */
function _subscribeToNewMessages(currentEngagement) {
  return currentEngagement.messagePosted((oscMessage) => {
    if (oscMessage.Sender === constants.senders.END_USER) {
      const endUserWidgetInfo = getEndUserWidgetInfo(oscMessage.Body);
      if (endUserWidgetInfo) {
        log.info("Detected an end-user widget info message AFTER plugin initialization");
        if (endUserQualityRatingConfig.ratingTypeId !== endUserWidgetInfo.ratingTypeId) {
          log.warn(
            `Changing end-user quality rating config from "${endUserQualityRatingConfig.ratingTypeId}" to "${endUserWidgetInfo.ratingTypeId}" based on end-user widget info message`
          );
          endUserQualityRatingConfig.ratingTypeId = endUserWidgetInfo.ratingTypeId;
        }

        if (endUserQualityRatingConfig.ratingInstructions !== "") {
          return _postOscMessage(
            "[LIO] Translation Quality Rating Instructions: " + endUserQualityRatingConfig.ratingInstructions
          );
        }
        return Promise.resolve();
      }
    }

    let alreadyHandled = false;
    const fixedContent = fixPlaintext(oscMessage.Body);
    /* agent messages sent from the extension already have an LIO message object. depending on
      whether we translated them, the sent text could be the original or translation. */
    if (oscMessage.Sender === constants.senders.AGENT) {
      if (getTranslationIdAndMessage(oscMessage.Body) !== undefined) {
        // the message has our translation ID prefixed, so we know it was already handled
        alreadyHandled = true;
      } else if (/^\[LIO\] Translation Quality Rating Instructions: .*$/.test(oscMessage.Body)) {
        // the message matches our out-going message with the translation quality rating instructions
        alreadyHandled = true;
      } else {
        /* we don't know the exact time that the OSC message post event goes through, so we check
          for duplicates via a combination of content (normalized for linebreak and spacing
          inconsistencies) and approximate post time (to avoid hiding legitimate duplicates) */
        const matchableContent = removeSpaces(fixedContent);
        alreadyHandled = allMessages
          .filter((message) => message.sender === constants.senders.AGENT)
          .some(
            (agentMessage) =>
              // eslint-disable-next-line implicit-arrow-linebreak
              (matchableContent === removeSpaces(agentMessage.text) ||
                matchableContent === removeSpaces(agentMessage.translatedText)) &&
              Math.abs(oscMessage.PostTime.valueOf() - agentMessage.timestamp.valueOf()) <
                constants.messageTimestampVariation * 1000
          );
      }
    }

    if (alreadyHandled) {
      return Promise.resolve();
    }

    const commandData = getCommandData(oscMessage.Body);
    if (commandData) {
      // handle this command now, as it must be for a message already parsed/rendered
      if (handleCommand(commandData)) {
        return Promise.resolve();
      }
    }

    // customer messages are the only ones that have a status and are eligible for quality rating by the agent
    const message =
      oscMessage.Sender === constants.senders.END_USER
        ? new Message(
            oscMessage.Sender,
            oscMessage.PostTime,
            fixedContent,
            constants.statuses.awaiting,
            profileConfig.qualityRatingInstructions,
            profileConfig.qualityRatingType
          )
        : new Message(oscMessage.Sender, oscMessage.PostTime, fixedContent);
    _displayMessage(message);

    if (oscMessage.Sender === constants.senders.END_USER) {
      return _translateAndContinue(message);
    }

    return Promise.resolve();
  });
}

/** display and attempt translation for any messages sent/received before loading the extension */
function _loadAndDisplayPreviousMessages(currentEngagement) {
  return currentEngagement.getMessages().then((messageList) => {
    const preLoadCommands = [];

    const messagePromises = messageList.map((oscMessage) => {
      if (oscMessage.Sender === constants.senders.END_USER) {
        const endUserWidgetInfo = getEndUserWidgetInfo(oscMessage.Body);
        if (endUserWidgetInfo) {
          endUserQualityRatingConfig.ratingTypeId = endUserWidgetInfo.ratingTypeId;

          if (endUserQualityRatingConfig.ratingInstructions !== "") {
            return _postOscMessage(
              "[LIO] Translation Quality Rating Instructions: " + endUserQualityRatingConfig.ratingInstructions
            );
          }
          return Promise.resolve();
        }
      }

      // avoid duplication if a new message triggers the subscribe even before we've gotten to it
      const alreadyHandled = allMessages.some(
        (message) => oscMessage.Sender === message.Sender && String(oscMessage.PostTime) === String(message.timestamp)
      );

      if (alreadyHandled) {
        return Promise.resolve();
      }

      let initialStatus = "";
      if (oscMessage.Sender === constants.senders.END_USER) {
        initialStatus = constants.statuses.awaiting;
      }

      let messageBody = oscMessage.Body;
      let translationId = 0;

      // pre-loaded agent messages might include the translation identifier
      // if so, we want to capture and remove it before we display the message
      const messageData = getTranslationIdAndMessage(messageBody);
      if (oscMessage.Sender === constants.senders.AGENT && messageData !== undefined) {
        messageBody = messageData.message;
        translationId = messageData.translationId;
      } else {
        const commandData = getCommandData(messageBody);
        if (commandData) {
          // attempt to handle the command message now, as we likely already processed the corresponding message
          if (!handleCommand(commandData, true)) {
            // save this message's command data to try to handle after ALL messages have been parsed/rendered
            preLoadCommands.push(commandData);
          }
          return Promise.resolve();
        }
      }

      const message = new Message(
        oscMessage.Sender,
        oscMessage.PostTime,
        fixPlaintext(messageBody),
        initialStatus,
        profileConfig.qualityRatingInstructions,
        profileConfig.qualityRatingType,
        translationId
      );
      _displayMessage(message, false, true);

      if (oscMessage.Sender === constants.senders.END_USER) {
        return _translateAndContinue(message);
      }

      return Promise.resolve();
    });

    // now handle any remaining commands that were received while processing pre-load messages
    return Promise.allSettled(messagePromises).then(() => {
      preLoadCommands.forEach((commandData) => {
        if (!handleCommand(commandData, true)) {
          log.warn("Invalid pre-load command message detected" + JSON.stringify(commandData));
        }
      });
    });
  });
}

/** attempt translation, log errors, and resolve regardless */
function _translateAndContinue(message) {
  return lioRestManager
    .getTranslation(message)
    .catch((error) => log.error(error))
    .finally(() => Promise.resolve());
}

/** attempt to retrieve and post an autoresponder from LIO if applicable */
function _sendAutoresponder() {
  if (profileConfig.sendAutoresponder) {
    log.info("Custom Autoresponder is enabled.");
    return lioRestManager
      .getAutoresponder()
      .then((autoresponse) => _createOutgoingMessage(autoresponse))
      .then((message) => {
        _postOscMessage(message.text)
          .then(() => message.addStatus("SUCCESS: Autoresponder sent."))
          .catch(() => message.addStatus(constants.statuses.postFailed + " Autoresponder was not sent."))
          .finally(() => Promise.resolve());
      })
      .catch((error) => {
        log.warn("Autoresponder failed: " + error);
        const message = new Message(
          constants.senders.SYSTEM,
          new Date(),
          "There is no Language I/O autoresponder set up for this customer language."
        );
        _displayMessage(message);
        return Promise.resolve();
      });
  } else {
    log.info("Custom autoresponder is disabled.");
    return Promise.resolve();
  }
}

/* MESSAGE HANDLING */

/** get the content in the agent text editor and replace variables, translate if necessary,
 * and attempt to post */
async function processAgentMessage() {
  agentTextEditor.disable();

  const needsTranslation = document.getElementById("translate_checkbox").checked;
  const notSentText = " No message has been sent.";

  const agentText = agentTextEditor.editor.value.replace(/(<br>)+$/g, "");
  if (agentText === "") {
    agentTextEditor.enable();
    return;
  }

  const initialStatus = needsTranslation ? constants.statuses.awaiting + notSentText : constants.statuses.sending;
  const message = await _createOutgoingMessage(agentText, initialStatus);

  let translationId;
  let textToSend = message.text;
  if (needsTranslation) {
    try {
      const translation = await lioRestManager.getTranslation(message);
      translationId = translation.translationId;
      textToSend = translation.translatedText;
    } catch (error) {
      message.addStatus(message.getFullStatus() + notSentText);
      agentTextEditor.enable();
      return;
    }
  }

  _postOscMessage(textToSend, translationId)
    .then(() => {
      agentTextEditor.editor.value = "";
      if (!needsTranslation) {
        message.addStatus(constants.statuses.sent);
      }
    })
    .catch(() => message.addStatus(constants.statuses.postFailed + notSentText))
    .finally(() => agentTextEditor.enable());
}

/** display the message, scroll it into view, and set up scrolling for the translation */
function _displayMessage(message, isSubject = false, isPreLoad = false) {
  const id = message.id.toString();

  const newMessage = document.createElement("div");
  newMessage.innerHTML = message.getDisplayHtml();

  let messageContainer = document.getElementById("post_load_messages");
  if (isSubject) {
    messageContainer = document.getElementById("subject_message");
  } else if (isPreLoad) {
    messageContainer = document.getElementById("pre_load_messages");
  }
  messageContainer.appendChild(newMessage);

  // setup event handlers for each message
  window.addEventListener("RenderMessage_" + id, () => {
    const messageElement = document.getElementById("lio_chat_message_" + id);
    messageElement.outerHTML = message.getDisplayHtml();
  });
  window.addEventListener("ScrollMessageIntoView_" + id, () => {
    Message.scrollMessageIntoView(id);
  });
  window.addEventListener("CreateMessageRating_" + id, (event) => {
    lioRestManager.createTranslationRating(event.detail).then((rating) => {
      message.setRatingInfo(rating);
    });
  });
  window.addEventListener("UpdateMessageRating_" + id, (event) => {
    lioRestManager.updateTranslationRating(event.detail).then((rating) => {
      message.setRatingInfo(rating);
    });
  });

  // hack - if the scroll isn't delayed, autoscroll fails for untranslated agent messages.
  delay(() => Message.scrollMessageIntoView(id), 50);
}

/**
 * Returns the sender of the original message for the given command
 * @param {constants.senders} command.source the source of the command
 * @param {constants.commandTypes} command.type the type of command
 * @param {string} command.action the action string for the command
 * @returns {constants.senders|null} the sender of the original message, or null
 */
function _getOriginalMessageSenderForCommand(command) {
  let originalMessageSender = null;

  // eslint-disable-next-line default-case
  switch (command.type) {
    case constants.commandTypes.qualityRating:
      if (command.source === constants.senders.END_USER) {
        // an end-user quality rating command is flagging an AGENT message
        originalMessageSender = constants.senders.AGENT;
      }
      break;
  }

  return originalMessageSender;
}

/**
 * Checks if a command (with an `action` and `translationId`) is valid
 * @param {constants.senders} command.source the source of the command
 * @param {constants.commandTypes} command.type the type of command
 * @param {string} command.action the action string for the command
 * @param {string|number} command.translationId the ID of the translation of the message for the the command
 * @returns {Message|null} the Message object for the given command
 */
function getMessageForCommand(command) {
  log.info("Getting message for command", command);

  // first check if the action is recognized
  if (!["flagged", "un-flagged"].includes(command.action.toLowerCase())) {
    return null;
  }

  // then check if any messages match the translation ID
  let messages = allMessages.filter((message) => message.translationId === command.translationId, 10);

  // when we know the message had to come from a specific sender, filter by that too
  const originalMessageSender = _getOriginalMessageSenderForCommand(command);
  if (originalMessageSender !== null) {
    messages = messages.filter((message) => message.sender === originalMessageSender);
  }

  // return nothing if there isn't exactly one match to avoid ambiguity or abuse
  if (messages.length !== 1) {
    return null;
  }

  return messages[0];
}

/**
 * Handles a command (with an `action` and `translationId`)
 * @param {constants.senders} command.source the source of the command
 * @param {constants.commandTypes} command.type the type of command
 * @param {string} command.action the action string for the command
 * @param {string|number} command.translationId the ID of the translation of the message for the the command
 * @param {boolean} isPreLoad indicates if this command came while processing preload messages
 * @returns {boolean} if the command was handled successfully
 */
function handleCommand(command, isPreLoad = false) {
  log.info(
    `Handling "${command.action}" command for ${isPreLoad ? "preloaded " : ""}translation ID ${command.translationId}`
  );

  const message = getMessageForCommand(command);
  if (!message) {
    log.error(
      `Failed to handle "${command.action}" command: no message found for translation ID ${command.translationId}`
    );
    return false;
  }

  switch (command.action.toLowerCase()) {
    case "flagged":
      _displayNotificationForMessage(
        profileConfig.endUserFlaggedMessageText.replace("{customer}", names.customer),
        message,
        isPreLoad
      );
      toggleTranslationFlagRating(message.id, message.translationId, message.ratingId, message.ratingPoints);
      return true;
    case "un-flagged":
      _displayNotificationForMessage(
        profileConfig.endUserUnFlaggedMessageText.replace("{customer}", names.customer),
        message,
        isPreLoad
      );
      toggleTranslationFlagRating(message.id, message.translationId, message.ratingId, message.ratingPoints);
      return true;
    default:
      return false;
  }
}

/**
 * Displays a notification to the agent with a button to scroll the related message into view
 * @param {string} content HTML content to show
 * @param {Message} message a Message object related to the content
 * @param {boolean} isPreLoad indicates if this notification goes into the preload section or not
 */
function _displayNotificationForMessage(content, message, isPreLoad = false) {
  const newNotification = document.createElement("div");
  newNotification.innerHTML = `
  <div class="${constants.senders.SYSTEM} message">
    <div class="${constants.senders.SYSTEM} text">
      ${content}
      ${
        message !== undefined
          ? `<button class="${constants.senders.SYSTEM} button scrollToMsg" onclick="Message.scrollMessageIntoView('${message.id}')">[${profileConfig.goToMessageText}]</button>`
          : ``
      }
    </div>
  </div>
  `;

  const messageContainer = document.getElementById(isPreLoad ? "pre_load_messages" : "post_load_messages");
  messageContainer.appendChild(newNotification);
}

/** replace variables, create, and display a new agent message */
async function _createOutgoingMessage(text, initialStatus) {
  const fixedText = await processVariables(text);
  const message = new Message(constants.senders.AGENT, new Date(), fixedText, initialStatus);
  _displayMessage(message);
  return message;
}

/** attempt to send the message to the customer */
async function _postOscMessage(messageContent, translationId = null) {
  const message =
    (endUserQualityRatingConfig.ratingTypeId > 0 && translationId !== null ? `[LIO::${translationId}] ` : "") +
    messageContent;
  return new Promise((resolve, reject) => {
    oscObjects.oracleChatClient
      .getAgentSession()
      .then((agentSession) => agentSession.getCurrentEngagement())
      .then((currentEng) => currentEng.postMessage(message))
      .then(() => resolve())
      .catch(() => reject());
  });
}
