/* global allMessages, constants, log, names, toggleElementClass */

// eslint-disable-next-line no-unused-vars
class Message {
  /** data + html generation for individual messages
   * @param {string} sender AGENT, END_USER, SUBJECT, SYSTEM, or ENGAGEMENT
   * @param {Date} timestamp OSC date or Date for the time the message was sent/received
   * @param {string} text html or plaintext message content
   * @param {string} [initialStatus] starting status including type (ex: 'PENDING: Awaiting translation.')
   * @param {string} [qualityRatingInstructions] the instructions to display with the quality rating controls
   * @param {number} [ratingTypeId] the type of quality rating system being used
   * @param {number} [translationId] the translation id
   */
  constructor(
    sender,
    timestamp,
    text,
    initialStatus = "",
    qualityRatingInstructions = "Rate this translation",
    ratingTypeId = 0,
    translationId = 0
  ) {
    this.id = Math.round(timestamp.getTime() / 1000) * 1000;
    this.sender = sender;
    this.timestamp = timestamp;

    this.qualityRatingInstructions = qualityRatingInstructions;
    this.ratingTypeId = ratingTypeId;
    this.ratingId = 0;
    this.ratingPoints = null;

    if (sender === constants.senders.AGENT) {
      this.name = names.agent;
    } else if (sender === constants.senders.END_USER) {
      this.name = names.customer;
    } else if (sender === constants.senders.SUBJECT) {
      // treating the subject as a special customer message
      this.sender = constants.senders.END_USER;
      this.name = names.subject;
    } else {
      // map SYSTEM + ENGAGEMENT + unexpected values to SYSTEM
      this.sender = constants.senders.SYSTEM;
      this.name = "";
    }
    this.text = text;

    this.statusType = "";
    this.statusMessage = "";
    if (initialStatus.length > 0) {
      this.addStatus(initialStatus, true);
    }

    this.translatedText = "";
    this.rnResponseProjectId = 0;
    this.translationId = translationId;
    this.wordCount = 0;

    allMessages.push(this);
  }

  /** convert the timestamp from OSC into the string format they use in their display
   * @param {*} oscTimestamp timestamp returned from the BUI API calls
   */
  static _getFormattedDate(oscTimestamp) {
    const dt = new Date(oscTimestamp);
    const rawHour = dt.getHours();
    const suffix = rawHour >= 12 ? "PM" : "AM";
    let hour = rawHour > 12 ? rawHour - 12 : rawHour;
    hour = hour === "00" ? 12 : hour;
    const minute = dt.getMinutes().toString().padStart(2, "0");
    return hour + ":" + minute + " " + suffix;
  }

  render(scrollIntoView = false) {
    dispatchEvent(new CustomEvent("RenderMessage_" + this.id));
    if (scrollIntoView) {
      dispatchEvent(new CustomEvent("ScrollMessageIntoView_" + this.id));
    }
  }

  /** set info returned from the Language I/O server, and indicate that the object has been updated
   * @param {string} translatedText content returned from the Language I/O server
   * @param {int} rnResponseProjectId project id used in LNThreads after the chat
   * @param {int} translationId translation id used in LNThreads after the chat
   * @param {int} wordCount number of source words, used in LNThreads after the chat
   */
  addTranslationInfo(translatedText, rnResponseProjectId, translationId, wordCount) {
    this.translatedText = translatedText;
    this.rnResponseProjectId = rnResponseProjectId;
    this.translationId = translationId;
    this.wordCount = wordCount;

    this.render(true);
  }

  /**
   * set properties related to quality rating and (re)render message when applicable
   * @param {object} config
   */
  setQualityRatingConfig(config) {
    let updated = false;

    if (Object.prototype.hasOwnProperty.call(config, "instructions")) {
      if (this.qualityRatingInstructions !== config.instructions) {
        this.qualityRatingInstructions = config.instructions;
        updated = true;
      }
    }

    if (Object.prototype.hasOwnProperty.call(config, "type")) {
      if (this.ratingTypeId !== config.type) {
        // TODO: if the rating type changes, then the rating value might need to be changed/reset to match...
        this.ratingTypeId = config.type;
        updated = true;
      }
    }

    if (updated) {
      this.render(false);
    }
  }

  /**
   * set info returned from the Language I/O server for a translation quality rating and re-render the message
   * @param {object} rating rating response data from the Language I/O server
   */
  setRatingInfo({ ratingId, ratingPoints, ratingTypeId }) {
    this.ratingId = ratingId;
    this.ratingPoints = ratingPoints;
    this.ratingTypeId = ratingTypeId;

    this.render(false);
  }

  /* STATUS MANAGEMENT */

  /**
   * splits the status into a type and message so we can turn the type into an appropriate button
   * @param {string} statusMessage full message, starting with type, ex. 'SUCCESS: Translation complete.'
   * @param {bool} initial true if the status is a part of the initial message
   */
  addStatus(statusMessage, initial = false) {
    if (statusMessage.length > 1) {
      this.statusType = "INFO";
      this.statusMessage = statusMessage;

      const statusTypes = [constants.statuses.PENDING, constants.statuses.SUCCESS, constants.statuses.ERROR];
      statusTypes.forEach((statusType) => {
        if (statusMessage.startsWith(statusType)) {
          this.statusType = statusType;
          this.statusMessage = statusMessage.replace(`${statusType}: `, "");
        }
      });
      if (statusMessage.startsWith(constants.statuses.ERROR)) {
        log.warn(statusMessage);
      }

      if (!initial) {
        this.render(true);
      }
    }
  }

  /**
   * @returns {string} the status type + message text
   */
  getFullStatus() {
    return `${this.statusType}: ${this.statusMessage}`;
  }

  /* HTML GENERATION */

  /**
   * scroll the message into view if it isn't already
   */
  static scrollMessageIntoView(id) {
    document.getElementById(`lio_chat_message_${id}`).scrollIntoView({
      behavior: "smooth",
      block: "nearest",
      inline: "nearest",
    });
  }

  /**
   * @returns {string} html to represent this message in the incident's system thread transcript
   */
  getTranscriptHtml() {
    // agent-language content, if it exists, for the incident transcript
    let agentLangText = this.text;
    if (this.sender === constants.senders.END_USER) {
      if (this.translatedText.length > 0) {
        agentLangText = this.translatedText;
      }
    }

    if (this.sender === constants.senders.AGENT || this.sender === constants.senders.END_USER) {
      return `<i>${this.name}</i>: ${agentLangText}<br/>`;
    } else {
      // system messages don't need to be included in our transcript
      return "";
    }
  }

  /**
   * @returns {string} html to display the whole message during the chat
   */
  getDisplayHtml() {
    // system messages are the simplest case - no status/sender/etc. display
    if (this.sender === constants.senders.SYSTEM) {
      return `<div class="${this.sender} message" id="lio_chat_message_${this.id}">
        <div class="${this.sender} text">
          ${this.text}
        </div>
      </div>`;
    } else {
      // full message html containing sender, timestamp (unless subject), status, and text(s)
      let timestamp = `<span class="timestamp">
          ${Message._getFormattedDate(this.timestamp)}
        </span>`;
      if (this.name === names.subject) {
        timestamp = "";
      }
      return `<div class="${this.sender} message" id="lio_chat_message_${this.id}">
        <div class="statusbar">
          <span class="sender_name">${this.name}</span>
          ${this._getContentToggleHtml()}
          ${this._getStatusHtml()}
          ${timestamp}
        </div>
        <div class="text ${this.statusType}">
          ${this._getContentHtml()}
          ${this._getQualityRatingHtml()}
        </div>
      </div>`;
    }
  }

  /**
   * @returns {string} html to display the status: a type button + toggle-able details
   */
  _getStatusHtml() {
    if (this.statusMessage.length > 0) {
      return `<span class="status container">
        <button class="status type ${this.statusType}" 
            onclick="_toggleStatus('${this.id}')">
          ${this.statusType}
        </button>
        <span class="status message collapsed" id="status_message_${this.id}">
          ${this.statusMessage}
        </span>
      </span>`;
    } else {
      return "";
    }
  }

  /**
   * @returns {string} html showing the agent-language content and toggle-able customer-language content
   */
  _getContentHtml() {
    /* original and translated text content */
    if (this.sender === constants.senders.AGENT) {
      return `<div class="original_text">
        ${this.text}
      </div>
      <div class="translated_text collapsed" id="toggleable_text_${this.id}">
        ${this.translatedText}
      </div>`;
    } else {
      // else it's an END_USER message - we already returned for other types
      return `<div class="translated_text">
        ${this.translatedText}
      </div>
      <div class="original_text collapsed" id="toggleable_text_${this.id}">
        ${this.text}
      </div>`;
    }
  }

  /**
   * @returns {string} html for the customer-language content view toggle, if applicable
   */
  _getContentToggleHtml() {
    if (
      this.sender === constants.senders.END_USER ||
      (this.sender === constants.senders.AGENT && this.translatedText)
    ) {
      return `<button class="translation_toggle ${this.sender}"
          id="toggle_text_${this.id}" onClick="_toggleCustomerLanguageView(${this.id})">
        </button>`;
    } else {
      return "";
    }
  }

  _getQualityRatingHtml() {
    if (!this.translationId && !this.ratingId) {
      // don't show the rating control(s) until there's something to rate
      return "";
    }

    switch (this.ratingTypeId) {
      case 1:
        const points = parseInt(this.ratingPoints, 10);
        const ratingClass = Number.isNaN(points) ? "rating_points_none" : "rating_points_" + points;
        const classes = "translation_quality_rating rating_type_flag " + ratingClass;
        const id = `translation_rating_${this.id}_flag`;
        if (this.sender === constants.senders.END_USER) {
          const params = `${this.id}, ${this.translationId}, ${this.ratingId}, ${this.ratingPoints}`;
          return `<button class="${classes}" id="${id}" onClick="toggleTranslationFlagRating(${params})" title="${this.qualityRatingInstructions}"></button>`;
        } else {
          return `<span class="${classes} static" id="${id}"></span>`;
        }
      case 0:
      default:
        return "";
    }
  }
}

/* MESSAGE BUTTON FUNCTIONALITY */

/**
 * toggle the status message view
 * @param {int} id message.id of the message whose status view should be toggled
 */
/* eslint-disable-next-line no-unused-vars */ // used by status buttons
function _toggleStatus(id) {
  const fullStatus = document.getElementById("status_message_" + id);
  toggleElementClass(fullStatus, "collapsed");
}

/**
 * toggle the customer language content view
 * @param {int} id id of the message to toggle
 */
/* eslint-disable-next-line no-unused-vars */ // used by status buttons
function _toggleCustomerLanguageView(id) {
  const button = document.getElementById("toggle_text_" + id);
  toggleElementClass(button, "hide_text");
  const text = document.getElementById("toggleable_text_" + id);
  if (!toggleElementClass(text, "collapsed")) {
    Message.scrollMessageIntoView(id);
  }
}

/**
 * toggle the translation's quality rating
 * @param {number} messageId message.id of the message whose translation quality is being rated
 * @param {number} translationId LIO ID of the translation being rated
 * @param {number} [ratingId] optional existing LIO rating ID (for updating/removing a rating)
 * @param {number} [currentRatingPoints] optional current rating points/value
 */
/* eslint-disable-next-line no-unused-vars */ // used by rating flag button
function toggleTranslationFlagRating(messageId, translationId, ratingId, currentRatingPoints) {
  let eventName = "";
  const detail = {
    ratingTypeId: 1,
  };

  const ratingIdInt = parseInt(ratingId, 10);
  if (ratingIdInt > 0) {
    eventName = "UpdateMessageRating";
    detail.ratingId = ratingIdInt;
    detail.ratingPoints = currentRatingPoints === 1 ? 0 : 1;
  } else {
    eventName = "CreateMessageRating";
    detail.translationId = translationId;
    detail.ratingPoints = 1;
  }

  dispatchEvent(
    new CustomEvent(eventName + "_" + messageId, {
      detail,
    })
  );
}
