/* global constants, oscObjects, log, LanguageIOError, fixPlaintext */

/* eslint-disable prefer-promise-reject-errors */ /* we only want to wrap a string in an Error
object once to preserve the stack trace but also prevent multiple 'Error:' titles. */

/* eslint-disable-next-line no-unused-vars */ // used in other files
class OscRestManager {
  /**
   * handles all REST calls to OSC
   * @param {string} sessionToken authentication token from BUI API
   */
  constructor(sessionToken) {
    this._headers = {
      "Content-Type": "application/json",
      "OSvC-CREST-Application-Context": "Language I/O BUI Chat REST Call",
      Authorization: "Session " + sessionToken,
    };

    this._baseUrl = oscObjects.globalContext.getInterfaceServiceUrl("REST") + "/connect/v1.4/";
    this._interfaceId = oscObjects.globalContext.getInterfaceId();
    this._globalContext = oscObjects.globalContext;

    this._accountNames = {};
    this._contactNames = {};
  }

  /**
   * wrapper for fetch to handle OSC errors, auth fails, fetch fails, and parsing fails consistently
   * @param {string} fullUrl the full url used in the call
   * @param {string} method ex. GET, POST
   * @param {*} body json body, if applicable
   * @param {boolean} secondAttempt whether we should refresh the session token and try again
   * @param {boolean} getSchema used to look up the custom language field name
   */
  async _safeFetch(fullUrl, method, body, secondAttempt = false, getSchema = false) {
    const parameters = {
      credentials: "same-origin",
      headers: this._headers,
      method,
    };
    parameters.headers.Accept = getSchema ? "application/schema+json" : "*/*";

    if (body) {
      parameters.body = JSON.stringify(body);
    }

    return new Promise((resolve, reject) => {
      fetch(fullUrl, parameters)
        /* eslint-disable-next-line consistent-return */ // we don't return an object for PATCHes
        .then((response) => {
          if (response.ok) {
            // OSC PATCHes don't return a json body, so just resolve with nothing
            if (method === "PATCH") {
              return resolve();
            } else {
              // return the json body for the caller to process
              response
                .json()
                .then((responseJson) => (responseJson ? resolve(responseJson) : reject("Empty response body.")))
                .catch(() => reject("Unable to parse OSC response body."));
            }
          } else {
            // if the call returned unauthorized, try again with a new session token - but only once
            /* eslint-disable-next-line no-lonely-if */ // does not apply
            if (response.status === 401 && !secondAttempt) {
              log.info("Unable to authenticate with OSC database. Trying again with a new session token.");
              oscObjects.globalContext
                .getSessionToken()
                .then((newSessionToken) => {
                  this._headers.Authorization = "Session " + newSessionToken;
                  return resolve(this._safeFetch(fullUrl, method, body, true));
                })
                .catch((error) => {
                  log.error(error);
                  return reject("Could not get new session token.");
                });
            } else {
              const errorText = "Could not communicate with OSC database.";
              response
                .json()
                .then((errorJson) => {
                  log.warn(new Error(`Fetch failed: ${response.status} ${response.statusText}\n${errorJson.detail}`));
                  reject(errorText);
                })
                .catch(() => reject(errorText));
            }
          }
        });
    });
  }

  /* GETTING EXTENSION CONFIGURATION FROM CUSTOM OBJECT(S) AND CUSTOM CONFIGURATION SETTINGS */

  /**
   * @returns {Promise<string>} string name used by OSC to look up the custom language field
   * @param {string} languageFieldDisplayName display name seen by the agent
   */
  async getCustomLanguageLookupName(languageFieldDisplayName) {
    const fullUrl = this._baseUrl + "metadata-catalog/incidents/customFields/c/";
    return this._safeFetch(fullUrl, "GET", "", false, true)
      .then((json) => {
        let match;
        /* if we can't find a match below, we want to fall into the error case */
        /* eslint-disable-next-line consistent-return */
        Object.keys(json.properties).forEach((key) => {
          if (json.properties[key].label === languageFieldDisplayName) {
            match = key.toString();
          }
        });
        if (match) {
          return match;
        } else {
          throw new Error();
        }
      })
      .catch((error) => {
        log.error(error);
        throw new LanguageIOError(
          `Couldn't get custom language field '${languageFieldDisplayName}'. Verify that the 'languageField' server configuration property is set correctly for your profile, or ${constants.contactText}`
        );
      });
  }

  /**
   * @returns {Promise<string>} name of the active agent
   */
  async getAgentName() {
    const fullUrl = `${this._baseUrl}accounts/${oscObjects.globalContext.accountId}`;
    return this._safeFetch(fullUrl, "GET")
      .catch(() => undefined)
      .then((json) => (json && json.displayName ? json.displayName : undefined));
  }

  /* CREATING AND GETTING THREADS */

  /**
   * create an LNThread, used for reporting after the chat
   * @param {int} incidentId id of the incident that will hold the LNThread
   * @param {int} threadType type (2 = translated agent message, 4 = translated customer message)
   * @param {Message} message the translated message we're creating the thread for
   */
  async createLNThread(incidentId, threadType, message) {
    const fullUrl = this._baseUrl + "linguistnow.LNThread";
    const body = {
      incidentID: {
        id: incidentId,
      },
      sourceLNProjectID: message.rnResponseProjectId,
      text: message.text,
      threadType,
      translationID: message.translationId,
      wordCount: message.wordCount,
      contentType: 3, // chat
      sourceThreadID: 0, // not used in chats
      sourceLNThreadID: 0, // not used in chats
      statusMessage: "Chat Message Translation",
    };
    log.info(body);
    return this._safeFetch(fullUrl, "POST", body);
  }

  /**
   * set the ln_status for the incident so it can be searched on later
   * @param {int} incidentId the id of the incident that will contain the transcript
   */
  async updateLNStatus(incidentId) {
    const fullUrl = `${this._baseUrl}incidents/${incidentId}`;
    const body = {
      customFields: {
        c: {
          ln_status: "Translated Chat Conversation",
        },
      },
    };
    return this._safeFetch(fullUrl, "PATCH", body).catch((error) => {
      const modError = error;
      modError.message = "Failed to update Language I/O Status: " + error.message;
      log.warn(modError);
    });
  }

  /* GETTING STANDARD TEXT */

  /**
   * @returns {Promise<string>} the standard text, if it exists
   * @param {string} hotkey text entered by the agent
   */
  async getStandardText(hotkey) {
    return this._getStandardTextId(hotkey)
      .then((id) => this._getStandardTextValue(id))
      .catch((error) => {
        log.warn(`Couldn't get standard text: ${error.message}`);
        return Promise.reject();
      });
  }

  /**
   * @returns {Promise<int>} the id for the hotkey
   */
  async _getStandardTextId(hotkey) {
    const fullUrl = `${this._baseUrl}queryResults?query=SELECT id FROM StandardContents WHERE HotKey = '${hotkey}'`;
    return this._safeFetch(fullUrl, "GET")
      .catch(() => {
        throw new Error(`'${hotkey}' doesn't exist`);
      })
      .then((json) => {
        if (json && json.status) {
          // we only get this field if its returning a query error
          throw new Error(json.status);
        } else {
          return json.items[0].rows[0][0]; // location of language value if successful
        }
      });
  }

  /**
   * @returns {Promise<string>} the value for the id
   */
  async _getStandardTextValue(id) {
    const fullUrl = `${this._baseUrl}standardContents/${id}/contentValues/1`;
    return this._safeFetch(fullUrl, "GET")
      .catch(() => {
        throw new Error(`id=${id} doesn't exist on interface=${this._interfaceId}`);
      })
      .then((json) => {
        if (json && json.status) {
          // we only get this field if its returning a query error
          throw new Error(json.status);
        } else {
          return fixPlaintext(json.value);
        }
      });
  }
}
