/* global oscObjects, constants */

/**
 * @param {string} content plaintext content with possible newlines
 * @returns {string} html content with newlines fixed
 */
/* eslint-disable-next-line no-unused-vars */ // used in main
function fixPlaintext(content) {
  // convert \n newlines to <br> tags so that they'll display properly in our extension + the BUI
  return content.replace(/(\n)/gi, "<br>");
}

/**
 * @param {string} content content with whitespace
 * @returns {string} content without whitespace
 */
/* eslint-disable-next-line no-unused-vars */ // used in main
function removeSpaces(str) {
  return str.replace(/\s+/g, "");
}

/**
 * Gets LIO command data from a message string
 * @param {string} content chat message content
 * @returns {object|undefined} command data or `undefined`
 */
/* eslint-disable-next-line no-unused-vars */ // used in main
function getCommandData(content) {
  // eventually this may have multiple RegEx's, so this function is rather verbose right now
  const qualityRatingMsgRE =
    /^\[LIO\] end-user (?<action>(?:un-)?flagged) message \{(?<translationId>\d+)\} translation$/i;
  const qualityRatingMsgResult = qualityRatingMsgRE.exec(content);
  if (qualityRatingMsgResult) {
    return {
      source: constants.senders.END_USER,
      type: constants.commandTypes.qualityRating,
      translationId: parseInt(qualityRatingMsgResult.groups.translationId, 10),
      action: qualityRatingMsgResult.groups.action,
    };
  }

  return undefined;
}

/**
 * Gets information about the end-user's widget from a message string
 * @param {string} content chat message content
 * @returns {object|undefined} end-user widget information or `undefined`
 */
/* eslint-disable-next-line no-unused-vars */ // used in main
function getEndUserWidgetInfo(content) {
  const widgetInfoRE =
    /^\[LIO\] LIOChatTranscript v(?<widgetVersion>[\d.]+) widget activated; translation_quality_rating_type: (?<ratingTypeId>\d)$/i;
  const widgetInfoResult = widgetInfoRE.exec(content);
  if (widgetInfoResult) {
    return {
      widgetVersion: widgetInfoResult.groups.widgetVersion,
      ratingTypeId: parseInt(widgetInfoResult.groups.ratingTypeId, 10),
    };
  }

  return undefined;
}

/**
 * Gets the translation ID from a message string
 * @param {string} content chat message content
 * @returns {object|undefined} translation ID and the message or `undefined`
 */
/* eslint-disable-next-line no-unused-vars */ // used in main
function getTranslationIdAndMessage(content) {
  const translationIdentifierRE = /^\[LIO::(?<translationId>\d+)\] (?<message>.*)/;
  const translationIdentifierResult = translationIdentifierRE.exec(content);
  if (translationIdentifierResult) {
    return {
      translationId: parseInt(translationIdentifierResult.groups.translationId, 10),
      message: translationIdentifierResult.groups.message,
    };
  }

  return undefined;
}

/**
 * replace variables and wrap them in notranslate spans so the translation engine doesn't make
 *  them invalid, ex. by adding a space between $ and the name of the variable
 * @param {string} content content that may have variables, i.e. from the agent
 * @returns {Promise<string>} html content with variables fixed
 */
/* eslint-disable-next-line no-unused-vars */ // used in lio-rest-manager
async function processVariables(content) {
  /* matches substrings that are preceded by whitespace/newline/beginning of string, starts with
    a '$', followed by at least one letter, and then any number of letters, numbers, and/or '_'. may
    be be broken up one or more times by '.', and/or (once) by '$', but only if there is a
    letter/number/_ before and after.
    includes content like: '$contacts.name.first', '$VaRiable', '$incidents.c$ln_status',
    '$valid_variable_0___'. */
  /*  regex explanation:
      (?:^|\s)          beg of string OR 1 whitespace, tab, or newline
      \$                1 literal $
      (?:[a-z]+[\w]*)   non-capture group, 1 repetition
        [a-z]+            at least one letter
        [\w]*             zero or more letters, numbers, or _s
      (?:[\.\$]?[\w])*  non-capture group, 0+ repetitions
        [\.\$]?           zero or one . or $
        [\w]              a letter, number, or underscore
      gmi               global, multiline, case insensitive */
  let fixedContent = content;
  const variableRegex = /(?:^|\s|&#160;|&nbsp;|>)(\$[a-z][\w]*(?:[.$]?[\w])*)/gim;
  const variables = [];
  let variableArray = variableRegex.exec(content);
  while (variableArray) {
    variables.push(variableArray[1]);
    variableArray = variableRegex.exec(content);
  }
  // eslint-disable-next-line no-restricted-syntax
  for (const variable of variables) {
    // eslint-disable-next-line no-await-in-loop
    const value = await _getVariableValue(variable);
    fixedContent = fixedContent.replace(variable, value);
  }
  return fixedContent;
}

/**
 * @returns {Promise<string>} html of the replaced (if possible) variable, wrapped in no-translate
 * @param {string} variable the name of the variable, ex. $contacts.name.first
 */
async function _getVariableValue(variable) {
  if (!_getVariableValue.cachedVariables) {
    _getVariableValue.cachedVariables = {};
  }
  return new Promise((resolve) => {
    const variableName = variable.replace(/^\$/, "");
    if (variableName in _getVariableValue.cachedVariables) {
      resolve(_getVariableValue.cachedVariables[variableName]);
    } else {
      oscObjects.extensionProvider.registerWorkspaceExtension((workspaceRecord) => {
        workspaceRecord
          .getFieldValues([variableName])
          .then((fieldDetails) => {
            let value;
            try {
              const rawValue = fieldDetails.getField(variableName).getLabel();
              value = rawValue === null ? "" : `<span class="notranslate">${rawValue}</span>`;
            } catch (error) {
              value = `<span class="notranslate">$${variableName}</span>`;
            }
            _getVariableValue.cachedVariables[variableName] = value;
            return resolve(value);
          })
          .catch(() => resolve(variable));
      });
    }
  });
}
